=== modified file 'NEWS'
--- NEWS	2006-09-21 17:00:00 +0000
+++ NEWS	2006-09-21 22:59:25 +0000
@@ -35,6 +35,11 @@
       which processes these requests. (Andrew Bennetts, Robert Collins, Martin
       Pool)
 
+    * New command ``bzr version-info`` which can be used to get a summary
+      of the current state of the tree. This is especially useful as part
+      of a build commands. See ``doc/version_info.txt`` for more information 
+      (John Arbash Meinel)
+
   BUG FIXES:
 
     * Raise WeaveFormatError rather than StopIteration when trying to read

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-09-20 17:38:34 +0000
+++ bzrlib/builtins.py	2006-09-21 22:59:25 +0000
@@ -2916,6 +2916,7 @@
 # we do need to load at least some information about them to know of 
 # aliases.  ideally we would avoid loading the implementation until the
 # details were needed.
+from bzrlib.cmd_version_info import cmd_version_info
 from bzrlib.conflicts import cmd_resolve, cmd_conflicts, restore
 from bzrlib.bundle.commands import cmd_bundle_revisions
 from bzrlib.sign_my_commits import cmd_sign_my_commits

=== added file 'bzrlib/cmd_version_info.py'
--- bzrlib/cmd_version_info.py	1970-01-01 00:00:00 +0000
+++ bzrlib/cmd_version_info.py	2006-09-21 16:33:39 +0000
@@ -0,0 +1,87 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Commands for generating snapshot information about a bzr tree."""
+
+from bzrlib import (
+    branch,
+    version_info_formats,
+    workingtree,
+    )
+from bzrlib.commands import Command
+from bzrlib.errors import BzrCommandError
+from bzrlib.option import Option
+
+
+def _parse_version_info_format(format):
+    """Convert a string passed by the user into a VersionInfoFormat.
+
+    This looks in the version info format registry, and if the format
+    cannot be found, generates a useful error exception.
+    """
+    try:
+        return version_info_formats.get_builder(format)
+    except KeyError:
+        formats = version_info_formats.get_builder_formats()
+        raise BzrCommandError('No known version info format %s.'
+                              ' Supported types are: %s'
+                              % (format, formats))
+
+
+class cmd_version_info(Command):
+    """Generate version information about this tree."""
+
+    takes_options = [Option('format', type=_parse_version_info_format,
+                            help='Select the output format'),
+                     Option('all', help='include all possible information'),
+                     Option('check-clean', help='check if tree is clean'),
+                     Option('include-history',
+                            help='Include the revision-history'),
+                     Option('include-file-revisions',
+                            help='Include the last revision for each file')
+                     ]
+    takes_args = ['location?']
+
+    encoding_type = 'exact'
+
+    def run(self, location=None, format=None,
+            all=False, check_clean=False, include_history=False,
+            include_file_revisions=False):
+
+        if location is None:
+            location = '.'
+
+        if format is None:
+            format = version_info_formats.get_builder(None)
+
+        wt = None
+        try:
+            wt = workingtree.WorkingTree.open_containing(location)[0]
+        except NoWorkingTree:
+            b = branch.Branch.open(location)
+        else:
+            b = wt.branch
+
+        if all:
+            include_history = True
+            check_clean = True
+            include_file_revisions=True
+
+        builder = format(b, working_tree=wt,
+                check_for_clean=check_clean,
+                include_revision_history=include_history,
+                include_file_revisions=include_file_revisions)
+        builder.generate(self.outf)

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-09-21 17:00:00 +0000
+++ bzrlib/tests/__init__.py	2006-09-21 22:59:25 +0000
@@ -1570,6 +1570,7 @@
                    'bzrlib.tests.test_urlutils',
                    'bzrlib.tests.test_versionedfile',
                    'bzrlib.tests.test_version',
+                   'bzrlib.tests.test_version_info',
                    'bzrlib.tests.test_weave',
                    'bzrlib.tests.test_whitebox',
                    'bzrlib.tests.test_workingtree',

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2006-09-13 06:45:03 +0000
+++ bzrlib/tests/blackbox/__init__.py	2006-09-20 14:51:03 +0000
@@ -88,6 +88,7 @@
                      'bzrlib.tests.blackbox.test_uncommit',
                      'bzrlib.tests.blackbox.test_update',
                      'bzrlib.tests.blackbox.test_upgrade',
+                     'bzrlib.tests.blackbox.test_version_info',
                      'bzrlib.tests.blackbox.test_versioning',
                      'bzrlib.tests.blackbox.test_whoami',
                      ]

=== added file 'bzrlib/tests/blackbox/test_version_info.py'
--- bzrlib/tests/blackbox/test_version_info.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_version_info.py	2006-09-21 16:48:22 +0000
@@ -0,0 +1,114 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Blackbox tests for version_info"""
+
+import os
+
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestVersionInfo(TestCaseWithTransport):
+
+    def test_invalid_format(self):
+        self.run_bzr('version-info', '--format', 'quijibo', retcode=3)
+
+    def create_branch(self):
+        wt = self.make_branch_and_tree('branch')
+
+        self.build_tree(['branch/a'])
+        wt.add('a')
+        wt.commit('adding a', rev_id='r1')
+
+        self.build_tree(['branch/b'])
+        wt.add('b')
+        wt.commit('adding b', rev_id='r2')
+
+        self.revisions = wt.branch.revision_history()
+
+    def test_basic(self):
+        self.create_branch()
+
+        txt = self.run_bzr('version-info', 'branch')[0]
+        self.assertContainsRe(txt, 'date:')
+        self.assertContainsRe(txt, 'build-date:')
+        self.assertContainsRe(txt, 'revno: 2')
+        self.assertContainsRe(txt, 'revision-id: ' + self.revisions[-1])
+
+    def test_all(self):
+        """'--all' includes clean, revision history, and file revisions"""
+        self.create_branch()
+        txt = self.run_bzr('version-info', 'branch',
+                           '--all')[0]
+        self.assertContainsRe(txt, 'date:')
+        self.assertContainsRe(txt, 'revno: 2')
+        self.assertContainsRe(txt, 'revision-id: ' + self.revisions[-1])
+        self.assertContainsRe(txt, 'clean: True')
+        self.assertContainsRe(txt, 'revisions:')
+        for rev_id in self.revisions:
+            self.assertContainsRe(txt, 'id: ' + rev_id)
+        self.assertContainsRe(txt, 'message: adding a')
+        self.assertContainsRe(txt, 'message: adding b')
+        self.assertContainsRe(txt, 'file-revisions:')
+        self.assertContainsRe(txt, 'path: a')
+        self.assertContainsRe(txt, 'path: b')
+
+    def test_clean(self):
+        """Test that --check-clean includes the right info"""
+        self.create_branch()
+
+        txt = self.run_bzr('version-info', 'branch',
+                           '--check-clean')[0]
+        self.assertContainsRe(txt, 'clean: True')
+
+        self.build_tree_contents([('branch/c', 'now unclean\n')])
+        txt = self.run_bzr('version-info', 'branch',
+                           '--check-clean')[0]
+        self.assertContainsRe(txt, 'clean: False')
+
+        txt = self.run_bzr('version-info', 'branch',
+                           '--check-clean', '--include-file-revisions')[0]
+        self.assertContainsRe(txt, 'revision: unversioned')
+
+        os.remove('branch/c')
+
+    def test_no_branch(self):
+        """Test that bzr defaults to the local working directory"""
+        self.create_branch()
+
+        txt1 = self.run_bzr('version-info', 'branch')[0]
+
+        os.chdir('branch')
+        txt2 = self.run_bzr('version-info')[0]
+        self.assertEqual(txt1, txt2)
+
+    def test_rio(self):
+        """Test that we can pass --format=rio"""
+        self.create_branch()
+
+        txt = self.run_bzr('version-info', 'branch')[0]
+        txt1 = self.run_bzr('version-info', '--format', 'rio', 'branch')[0]
+        txt2 = self.run_bzr('version-info', '--format=rio', 'branch')[0]
+        self.assertEqual(txt, txt1)
+        self.assertEqual(txt, txt2)
+
+    def test_python(self):
+        """Test that we can do --format=python"""
+        self.create_branch()
+
+        txt = self.run_bzr('version-info', '--format', 'python', 'branch')[0]
+
+        self.assertContainsRe(txt, 'version_info = {')

=== added file 'bzrlib/tests/test_version_info.py'
--- bzrlib/tests/test_version_info.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_version_info.py	2006-09-21 16:33:39 +0000
@@ -0,0 +1,217 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for version_info"""
+
+from cStringIO import StringIO
+import imp
+import os
+import sys
+
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.rio import read_stanzas
+
+from bzrlib.version_info_formats.format_rio import RioVersionInfoBuilder
+from bzrlib.version_info_formats.format_python import PythonVersionInfoBuilder
+
+
+class TestVersionInfo(TestCaseWithTransport):
+
+    def create_branch(self):
+        wt = self.make_branch_and_tree('branch')
+
+        self.build_tree(['branch/a'])
+        wt.add('a')
+        wt.commit('a', rev_id='r1')
+
+        self.build_tree(['branch/b'])
+        wt.add('b')
+        wt.commit('b', rev_id='r2')
+
+        self.build_tree_contents([('branch/a', 'new contents\n')])
+        wt.commit('a2', rev_id='r3')
+
+        return wt
+
+    def test_rio_version_text(self):
+        wt = self.create_branch()
+
+        def regen(**kwargs):
+            sio = StringIO()
+            builder = RioVersionInfoBuilder(wt.branch, working_tree=wt,
+                                            **kwargs)
+            builder.generate(sio)
+            val = sio.getvalue()
+            return val
+
+        val = regen()
+        self.assertContainsRe(val, 'build-date:')
+        self.assertContainsRe(val, 'date:')
+        self.assertContainsRe(val, 'revno: 3')
+        self.assertContainsRe(val, 'revision-id: r3')
+
+        val = regen(check_for_clean=True)
+        self.assertContainsRe(val, 'clean: True')
+
+        self.build_tree(['branch/c'])
+        val = regen(check_for_clean=True)
+        self.assertContainsRe(val, 'clean: False')
+        os.remove('branch/c')
+
+        val = regen(include_revision_history=True)
+        self.assertContainsRe(val, 'id: r1')
+        self.assertContainsRe(val, 'message: a')
+        self.assertContainsRe(val, 'id: r2')
+        self.assertContainsRe(val, 'message: b')
+        self.assertContainsRe(val, 'id: r3')
+        self.assertContainsRe(val, 'message: a2')
+
+    def test_rio_version(self):
+        wt = self.create_branch()
+
+        def regen(**kwargs):
+            sio = StringIO()
+            builder = RioVersionInfoBuilder(wt.branch, working_tree=wt,
+                                            **kwargs)
+            builder.generate(sio)
+            sio.seek(0)
+            stanzas = list(read_stanzas(sio))
+            self.assertEqual(1, len(stanzas))
+            return stanzas[0]
+
+        def get_one_stanza(stanza, key):
+            new_stanzas = list(read_stanzas(
+                                StringIO(stanza[key].encode('utf8'))))
+            self.assertEqual(1, len(new_stanzas))
+            return new_stanzas[0]
+
+        stanza = regen()
+        self.failUnless('date' in stanza)
+        self.failUnless('build-date' in stanza)
+        self.assertEqual(['3'], stanza.get_all('revno'))
+        self.assertEqual(['r3'], stanza.get_all('revision-id'))
+
+        stanza = regen(check_for_clean=True)
+        self.assertEqual(['True'], stanza.get_all('clean'))
+
+        self.build_tree(['branch/c'])
+        stanza = regen(check_for_clean=True, include_file_revisions=True)
+        self.assertEqual(['False'], stanza.get_all('clean'))
+
+        file_rev_stanza = get_one_stanza(stanza, 'file-revisions')
+        self.assertEqual(['a', 'b', 'c'], file_rev_stanza.get_all('path'))
+        self.assertEqual(['r3', 'r2', 'unversioned'],
+            file_rev_stanza.get_all('revision'))
+        os.remove('branch/c')
+
+        stanza = regen(include_revision_history=True)
+        revision_stanza = get_one_stanza(stanza, 'revisions')
+        self.assertEqual(['r1', 'r2', 'r3'], revision_stanza.get_all('id'))
+        self.assertEqual(['a', 'b', 'a2'], revision_stanza.get_all('message'))
+        self.assertEqual(3, len(revision_stanza.get_all('date')))
+
+        # a was modified, so it should show up modified again
+        self.build_tree(['branch/a', 'branch/c'])
+        wt.add('c')
+        wt.rename_one('b', 'd')
+        stanza = regen(check_for_clean=True, include_file_revisions=True)
+        file_rev_stanza = get_one_stanza(stanza, 'file-revisions')
+        self.assertEqual(['a', 'b', 'c', 'd'], file_rev_stanza.get_all('path'))
+        self.assertEqual(['modified', 'renamed to d', 'new', 'renamed from b'],
+                         file_rev_stanza.get_all('revision'))
+
+        wt.commit('modified', rev_id='r4')
+        wt.remove(['c', 'd'])
+        os.remove('branch/d')
+        stanza = regen(check_for_clean=True, include_file_revisions=True)
+        file_rev_stanza = get_one_stanza(stanza, 'file-revisions')
+        self.assertEqual(['a', 'c', 'd'], file_rev_stanza.get_all('path'))
+        self.assertEqual(['r4', 'unversioned', 'removed'],
+                         file_rev_stanza.get_all('revision'))
+
+    def test_python_version(self):
+        wt = self.create_branch()
+
+        def regen(**kwargs):
+            """Create a test module, import and return it"""
+            outf = open('test_version_information.py', 'wb')
+            try:
+                builder = PythonVersionInfoBuilder(wt.branch, working_tree=wt,
+                                                   **kwargs)
+                builder.generate(outf)
+            finally:
+                outf.close()
+            module_info = imp.find_module('test_version_information',
+                                          [os.getcwdu()])
+            tvi = imp.load_module('tvi', *module_info)
+            # Make sure the module isn't cached
+            sys.modules.pop('tvi', None)
+            sys.modules.pop('test_version_information', None)
+            # Delete the compiled versions, because we are generating
+            # a new file fast enough that python doesn't detect it
+            # needs to recompile, and using sleep() just makes the
+            # test slow
+            if os.path.exists('test_version_information.pyc'):
+                os.remove('test_version_information.pyc')
+            if os.path.exists('test_version_information.pyo'):
+                os.remove('test_version_information.pyo')
+            return tvi
+
+        tvi = regen()
+        self.assertEqual(3, tvi.version_info['revno'])
+        self.assertEqual('r3', tvi.version_info['revision_id'])
+        self.failUnless(tvi.version_info.has_key('date'))
+        self.assertEqual(None, tvi.version_info['clean'])
+
+        tvi = regen(check_for_clean=True)
+        self.assertEqual(True, tvi.version_info['clean'])
+
+        self.build_tree(['branch/c'])
+        tvi = regen(check_for_clean=True, include_file_revisions=True)
+        self.assertEqual(False, tvi.version_info['clean'])
+        self.assertEqual(['a', 'b', 'c'], sorted(tvi.file_revisions.keys()))
+        self.assertEqual('r3', tvi.file_revisions['a'])
+        self.assertEqual('r2', tvi.file_revisions['b'])
+        self.assertEqual('unversioned', tvi.file_revisions['c'])
+        os.remove('branch/c')
+
+        tvi = regen(include_revision_history=True)
+
+        rev_info = [(rev, message) for rev, message, timestamp, timezone
+                                   in tvi.revisions]
+        self.assertEqual([('r1', 'a'), ('r2', 'b'), ('r3', 'a2')], rev_info)
+
+        # a was modified, so it should show up modified again
+        self.build_tree(['branch/a', 'branch/c'])
+        wt.add('c')
+        wt.rename_one('b', 'd')
+        tvi = regen(check_for_clean=True, include_file_revisions=True)
+        self.assertEqual(['a', 'b', 'c', 'd'], sorted(tvi.file_revisions.keys()))
+        self.assertEqual('modified', tvi.file_revisions['a'])
+        self.assertEqual('renamed to d', tvi.file_revisions['b'])
+        self.assertEqual('new', tvi.file_revisions['c'])
+        self.assertEqual('renamed from b', tvi.file_revisions['d'])
+
+        wt.commit('modified', rev_id='r4')
+        wt.remove(['c', 'd'])
+        os.remove('branch/d')
+        tvi = regen(check_for_clean=True, include_file_revisions=True)
+        self.assertEqual(['a', 'c', 'd'], sorted(tvi.file_revisions.keys()))
+        self.assertEqual('r4', tvi.file_revisions['a'])
+        self.assertEqual('unversioned', tvi.file_revisions['c'])
+        self.assertEqual('removed', tvi.file_revisions['d'])
+
+

=== added directory 'bzrlib/version_info_formats'
=== added file 'bzrlib/version_info_formats/__init__.py'
--- bzrlib/version_info_formats/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/version_info_formats/__init__.py	2006-09-21 16:33:39 +0000
@@ -0,0 +1,202 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Routines for extracting all version information from a bzr branch."""
+
+import time
+
+from bzrlib.osutils import local_time_offset, format_date
+
+
+# This contains a map of format id => formatter
+# None is considered the default formatter
+_version_formats = {}
+
+def create_date_str(timestamp=None, offset=None):
+    """Just a wrapper around format_date to provide the right format.
+
+    We don't want to use '%a' in the time string, because it is locale
+    dependant. We also want to force timezone original, and show_offset
+
+    Without parameters this function yields the current date in the local
+    time zone.
+    """
+    if timestamp is None and offset is None:
+        timestamp = time.time()
+        offset = local_time_offset()
+    return format_date(timestamp, offset, date_fmt='%Y-%m-%d %H:%M:%S',
+                       timezone='original', show_offset=True)
+
+
+class VersionInfoBuilder(object):
+    """A class which lets you build up information about a revision."""
+
+    def __init__(self, branch, working_tree=None,
+                check_for_clean=False,
+                include_revision_history=False,
+                include_file_revisions=False,
+                ):
+        """Build up information about the given branch.
+        If working_tree is given, it can be checked for changes.
+
+        :param branch: The branch to work on
+        :param working_tree: If supplied, preferentially check
+            the working tree for changes.
+        :param check_for_clean: If False, we will skip the expense
+            of looking for changes.
+        :param include_revision_history: If True, the output
+            will include the full mainline revision history, including
+            date and message
+        :param include_file_revisions: The output should
+            include the explicit last-changed revision for each file.
+        """
+        self._branch = branch
+        self._working_tree = working_tree
+        self._check = check_for_clean
+        self._include_history = include_revision_history
+        self._include_file_revs = include_file_revisions
+
+        self._clean = None
+        self._file_revisions = {}
+        self._revision_history_info= []
+
+    def _extract_file_revisions(self):
+        """Extract the working revisions for all files"""
+
+        # Things seem clean if we never look :)
+        self._clean = True
+
+        if self._working_tree is not None:
+            basis_tree = self._working_tree.basis_tree()
+        else:
+            basis_tree = self._branch.basis_tree()
+
+        # Build up the list from the basis inventory
+        for info in basis_tree.list_files():
+            self._file_revisions[info[0]] = info[-1].revision
+
+        if not self._check or self._working_tree is None:
+            return
+
+        delta = self._working_tree.changes_from(basis_tree)
+
+        # Using a 2-pass algorithm for renames. This is because you might have
+        # renamed something out of the way, and then created a new file
+        # in which case we would rather see the new marker
+        # Or you might have removed the target, and then renamed
+        # in which case we would rather see the renamed marker
+        for (old_path, new_path, file_id,
+             kind, text_mod, meta_mod) in delta.renamed:
+            self._clean = False
+            self._file_revisions[old_path] = u'renamed to %s' % (new_path,)
+        for path, file_id, kind in delta.removed:
+            self._clean = False
+            self._file_revisions[path] = 'removed'
+        for path, file_id, kind in delta.added:
+            self._clean = False
+            self._file_revisions[path] = 'new'
+        for (old_path, new_path, file_id,
+             kind, text_mod, meta_mod) in delta.renamed:
+            self._clean = False
+            self._file_revisions[new_path] = u'renamed from %s' % (old_path,)
+        for path, file_id, kind, text_mod, meta_mod in delta.modified:
+            self._clean = False
+            self._file_revisions[path] = 'modified'
+
+        for path in self._working_tree.unknowns():
+            self._clean = False
+            self._file_revisions[path] = 'unversioned'
+
+    def _extract_revision_history(self):
+        """Find the messages for all revisions in history."""
+
+        # Unfortunately, there is no WorkingTree.revision_history
+        rev_hist = self._branch.revision_history()
+        if self._working_tree is not None:
+            last_rev = self._working_tree.last_revision()
+            assert last_rev in rev_hist, \
+                "Working Tree's last revision not in branch.revision_history"
+            rev_hist = rev_hist[:rev_hist.index(last_rev)+1]
+
+        repository =  self._branch.repository
+        repository.lock_read()
+        try:
+            for revision_id in rev_hist:
+                rev = repository.get_revision(revision_id)
+                self._revision_history_info.append(
+                    (rev.revision_id, rev.message,
+                     rev.timestamp, rev.timezone))
+        finally:
+            repository.unlock()
+
+    def _get_revision_id(self):
+        """Get the revision id we are working on."""
+        if self._working_tree is not None:
+            return self._working_tree.last_revision()
+        return self._branch.last_revision()
+
+    def generate(self, to_file):
+        """Output the version information to the supplied file.
+
+        :param to_file: The file to write the stream to. The output
+                will already be encoded, so to_file should not try
+                to change encodings.
+        :return: None
+        """
+        raise NotImplementedError(VersionInfoBuilder.generate)
+
+
+
+def register_builder(format, module, class_name):
+    """Register a version info format.
+
+    :param format: The short name of the format, this will be used as the
+        lookup key.
+    :param module: The string name to the module where the format class
+        can be found
+    :param class_name: The string name of the class to instantiate
+    """
+    if len(_version_formats) == 0:
+        _version_formats[None] = (module, class_name)
+    _version_formats[format] = (module, class_name)
+
+
+def get_builder(format):
+    """Get a handle to the version info builder class
+
+    :param format: The lookup key supplied to register_builder
+    :return: A class, which follows the VersionInfoBuilder api.
+    """
+    builder_module, builder_class_name = _version_formats[format]
+    module = __import__(builder_module, globals(), locals(),
+                        [builder_class_name])
+    klass = getattr(module, builder_class_name)
+    return klass
+
+
+def get_builder_formats():
+    """Get the possible list of formats"""
+    formats = _version_formats.keys()
+    formats.remove(None)
+    return formats
+
+
+register_builder('rio',
+                 'bzrlib.version_info_formats.format_rio',
+                 'RioVersionInfoBuilder')
+register_builder('python',
+                 'bzrlib.version_info_formats.format_python',
+                 'PythonVersionInfoBuilder')

=== added file 'bzrlib/version_info_formats/format_python.py'
--- bzrlib/version_info_formats/format_python.py	1970-01-01 00:00:00 +0000
+++ bzrlib/version_info_formats/format_python.py	2006-09-20 14:51:03 +0000
@@ -0,0 +1,103 @@
+# Copyright (C) 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""A generator which creates a python script from the current tree info"""
+
+import pprint
+
+from bzrlib.version_info_formats import (
+    create_date_str,
+    VersionInfoBuilder,
+    )
+
+
+# Header and footer for the python format
+_py_version_header = '''#!/usr/bin/env python
+"""This file is automatically generated by generate_version_info
+It uses the current working tree to determine the revision.
+So don't edit it. :)
+"""
+
+'''
+
+
+_py_version_footer = '''
+
+if __name__ == '__main__':
+    print 'revision: %(revno)d' % version_info
+    print 'nick: %(branch_nick)s' % version_info
+    print 'revision id: %(revision_id)s' % version_info
+'''
+
+
+class PythonVersionInfoBuilder(VersionInfoBuilder):
+    """Create a version file which is a python source module."""
+
+    def generate(self, to_file):
+        info = {'build_date':create_date_str()
+                  , 'revno':None
+                  , 'revision_id':None
+                  , 'branch_nick':self._branch.nick
+                  , 'clean':None
+                  , 'date':None
+        }
+        revisions = []
+
+        revision_id = self._get_revision_id()
+        if revision_id is None:
+            info['revno'] = 0
+        else:
+            info['revno'] = self._branch.revision_id_to_revno(revision_id)
+            info['revision_id'] = revision_id
+            rev = self._branch.repository.get_revision(revision_id)
+            info['date'] = create_date_str(rev.timestamp, rev.timezone)
+
+        if self._check or self._include_file_revs:
+            self._extract_file_revisions()
+
+        if self._check:
+            if self._clean:
+                info['clean'] = True
+            else:
+                info['clean'] = False
+
+        info_str = pprint.pformat(info)
+        to_file.write(_py_version_header)
+        to_file.write('version_info = ')
+        to_file.write(info_str)
+        to_file.write('\n\n')
+
+        if self._include_history:
+            self._extract_revision_history()
+            revision_str = pprint.pformat(self._revision_history_info)
+            to_file.write('revisions = ')
+            to_file.write(revision_str)
+            to_file.write('\n\n')
+        else:
+            to_file.write('revisions = {}\n\n')
+
+        if self._include_file_revs:
+            file_rev_str = pprint.pformat(self._file_revisions)
+            to_file.write('file_revisions = ')
+            to_file.write(file_rev_str)
+            to_file.write('\n\n')
+        else:
+            to_file.write('file_revisions = {}\n\n')
+
+        to_file.write(_py_version_footer)
+
+
+

=== added file 'bzrlib/version_info_formats/format_rio.py'
--- bzrlib/version_info_formats/format_rio.py	1970-01-01 00:00:00 +0000
+++ bzrlib/version_info_formats/format_rio.py	2006-09-20 15:11:19 +0000
@@ -0,0 +1,84 @@
+# Copyright (C) 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""A generator which creates a rio stanza of the current tree info"""
+
+from cStringIO import StringIO
+
+from bzrlib.rio import RioWriter, Stanza
+
+from bzrlib.version_info_formats import (
+    create_date_str,
+    VersionInfoBuilder,
+    )
+
+
+class RioVersionInfoBuilder(VersionInfoBuilder):
+    """This writes a rio stream out."""
+
+    def generate(self, to_file):
+        info = Stanza()
+        revision_id = self._get_revision_id()
+        if revision_id is not None:
+            info.add('revision-id', revision_id)
+            rev = self._branch.repository.get_revision(revision_id)
+            info.add('date', create_date_str(rev.timestamp, rev.timezone))
+            revno = str(self._branch.revision_id_to_revno(revision_id))
+        else:
+            revno = '0'
+
+        info.add('build-date', create_date_str())
+        info.add('revno', revno)
+
+        if self._branch.nick is not None:
+            info.add('branch-nick', self._branch.nick)
+
+        if self._check or self._include_file_revs:
+            self._extract_file_revisions()
+
+        if self._check:
+            if self._clean:
+                info.add('clean', 'True')
+            else:
+                info.add('clean', 'False')
+
+        if self._include_history:
+            self._extract_revision_history()
+            log = Stanza()
+            for (revision_id, message,
+                 timestamp, timezone) in self._revision_history_info:
+                log.add('id', revision_id)
+                log.add('message', message)
+                log.add('date', create_date_str(timestamp, timezone))
+            sio = StringIO()
+            log_writer = RioWriter(to_file=sio)
+            log_writer.write_stanza(log)
+            info.add('revisions', sio.getvalue())
+
+        if self._include_file_revs:
+            files = Stanza()
+            for path in sorted(self._file_revisions.keys()):
+                files.add('path', path)
+                files.add('revision', self._file_revisions[path])
+            sio = StringIO()
+            file_writer = RioWriter(to_file=sio)
+            file_writer.write_stanza(files)
+            info.add('file-revisions', sio.getvalue())
+
+        writer = RioWriter(to_file=to_file)
+        writer.write_stanza(info)
+
+

=== added file 'doc/version_info.txt'
--- doc/version_info.txt	1970-01-01 00:00:00 +0000
+++ doc/version_info.txt	2006-09-21 22:12:28 +0000
@@ -0,0 +1,57 @@
+==========================
+Using ``bzr version-info``
+==========================
+
+Overview
+========
+
+This document describes ways of using ``bzr version-info`` as part of a
+build routine to embed version information into a final project.
+
+
+Python Project
+==============
+
+TODO: Figure out how to attach into ``setup.py``
+
+
+If using a Makefile to build your project, you can generate the version
+information file as simply as::
+
+  library/_version.py:
+        bzr version-info --format=python > library/_version.py
+
+This generates a file which contains 3 dictionaries:
+
+  `version_info`: A dictionary containing the basic information about the
+                  current state.
+
+  revisions: A dictionary listing all of the revisions in the history of
+             the tree, along with the commit times and commit message.
+             This defaults to being empty unless ``--all`` or
+             ``--include-history`` is supplied. This is useful if you
+             want to track what bugfixes, etc, might be included in the
+             released version. But for many projects it is more
+             information than they need.
+
+  `file_revisions`: A dictionary listing the last-modified revision for
+                    all files in the project. This can be used similarly
+                    to how ``$Id$`` keywords are used in CVS controlled
+                    files. The last modified date can be determined by
+                    looking in the ``revisions`` map. This is also empty by
+                    default, and enabled only by ``--all`` or
+                    ``--include-file-revisions``.
+
+Check Clean
+===========
+
+Most information about the contents of the project can be cheaply
+determined by just reading the revision entry. However, it can be useful
+to know if the working tree was completely up-to-date when it was
+packaged, or if there was a local modification. By supplying either
+``--all`` or ``--check-clean``, ``bzr`` will inspect the working tree, and
+set the ``clean`` flag in ``version_info``, as well as set entries in
+``file_revisions`` as ``modified`` where appropriate.
+
+.. 
+   vim: tw=74 ft=rst spell spelllang=en_us

